@@ -413,6 +413,28 @@ Idents <- function(object, ... ) {
   UseMethod(generic = 'Idents<-', object = object)
 }
 
+#' Is an object global/persistent?
+#'
+#' Typically, when removing \code{Assay} objects from an \code{Seurat} object,
+#' all associated objects (eg. \code{DimReduc}, \code{Graph}, and \code{SeuratCommand} objects)
+#' are removed as well. If an associated object is marked as global/persistent,
+#' the associated object will remain even if its original assay was deleted
+#'
+#' @param object An object
+#' @param ... Arguments passed to other methods
+#'
+#' @return \code{TRUE} if the object is global/persistent otherwise \code{FALSE}
+#'
+#' @rdname IsGlobal
+#' @export IsGlobal
+#'
+#' @examples
+#' IsGlobal(pbmc_small[['pca']])
+#'
+IsGlobal <- function(object, ...) {
+  UseMethod(generic = 'IsGlobal', object = object)
+}
+
 #' Get JackStraw information
 #'
 #' @param object An object
@@ -784,9 +806,9 @@ RunICA <- function(object, ...) {
 #'
 #' For details about stored LSI calculation parameters, see
 #' \code{PrintLSIParams}.
-#' 
-#' @note RunLSI is being moved to Signac. Equivalent functionality can be 
-#' achieved via the Signac::RunTFIDF and Signac::RunSVD functions; 
+#'
+#' @note RunLSI is being moved to Signac. Equivalent functionality can be
+#' achieved via the Signac::RunTFIDF and Signac::RunSVD functions;
 #' for more information on Signac, please see
 #' \url{https://github.com/timoast/Signac}
 #'
@@ -800,7 +822,7 @@ RunLSI <- function(object, ...) {
   .Deprecated(
     new = 'Signac::RunTFIDF',
     msg = paste(
-      "RunLSI is being moved to Signac. Equivalent functionality can be", 
+      "RunLSI is being moved to Signac. Equivalent functionality can be",
       "achieved via the Signac::RunTFIDF and Signac::RunSVD functions; for",
       "more information on Signac, please see https://github.com/timoast/Signac"
     )
